;***************************************************************************
;
;    C C 4 0 0   -   D e m o n s t r a t i o n   B o a r d                   
;
;    File:      CC400DB.ASM   (Transceiver software)
;
;    Micro-controller: 
;               AVR AT90LS2343 or tiny11 (Internal oscillator)
;               Running from internal clock nominally 1MHz +/- 20%. 
;               
;    Author:    Developed for Chipcon Components AS by William Braathen @ ACTE Norway AS
;               
;    		Contact @ Chipcon Components AS:
;               Peder Martin Evjen (FAE), p.m.evjen@chipcon.no
;               (+47) 22-95 85 44 (office)
;		(+47) 22-95 83 01 (direct)
;    
;    Version:   1.0 - 6 oct. 1999
;
;
;***************************************************************************
    .nolist                       ; Don't generate list file from here
    .include "2343def.inc"        ; Include I/O and port definitions
    .list                         ; Generate list file from here

;***************************************************************************
;
;   DESCRIPTION
;   
;   General:  
;       This application demonstrates how to send a message and 
;       receive an acknowledge using the CC400 RF tranceiver chip from 
;       Chipcon Components and an AVR controller from Atmel.
;
;       The software contains driver routines to do background
;       transmission and receiption of data packages using RF. It also
;       contains a simple functions to encode and decode data in the 
;       transmission buffer (RAM). The main() program contains a simple 
;       state machine to handle key entries and allow the user to make
;       future real time applications with increased functionality.
;
;   Manual: 
;       When applying power for the first time, the CC400TRM will start in 
;       software idle mode with short RX listen periods each second. If no 
;       sync signal or key entries are detected for a timeout period (~2 min.), 
;       the CC400DB will enter power-down. Pushing "ON" button will reset 
;       CC400DB and re-enter RX polled mode.
;
;       If pressing TX button before the timeout period expires, the 
;       CC400DB will send a short RF message and go back to RX polled mode.
;
;       If the CC400DB is detecting activity during the short RX period, then
;       it will start continuos sync search, and when detecting sync signal,
;       wait for SOF (Start Of Frame). Data is reconstructed using external
;       interrupt and internal timer counter to measure pulse lengths. The
;       data clock rate is reconstructed during sync-search. The
;       reconstructed data (real data not manchester coded) is stored in RAM 
;       buffer until detecting EOP (End Of Package). The data sequence 
;       (except first byte) will finally be displayed as a blinking sequence
;       on the LED and the message (with different header) will be 
;       re-transmitted as an acknowledge (first byte different).
;
;   Power consumption:
;       Power down:           < 1.0uA   (Meashured to 0.2uA)
;       RX mode:                 18mA
;       TX Mode:                 30mA
;       IDLE mode average:   ~ 1.5 mA   (Most of the time sleep, wakeup @ timer int.)
;       RX polled mode average: 2.5mA
;
;   Init function:
;       The "ON"-(RX) button will only wakeup CC400DB from power down mode.
;
;   Main function:         
;       The main() routine contains a simple state machine to check button 
;       status of TX button and incoming data. 
;       Most of the time spent executing main() will be in SLEEP, Saving power and
;       waiting for next interrupt to wakeup the MCU from hardware-idle.
;
;   Timer interrupt:   
;       In general this timer will be used as the system clock with different
;       preset and prescaler values depending on mode of operation.
;       The timer is also used when displaying data blink sequence on the LED.
;
;   External interrupt:        
;       The external interrupt will be used while receiving RF data.
;       In power down mode the same I/0 pin will watch for low level interrupt
;       generated while user is pressing the "ON" button to wake-up CC400DB.  
;
;   Timing:
;       CC400 Full register reload takes ~1,5ms (measured to 1,42ms)                
;
;   Features and limitations:
;       The main advantage of this protocol/algorithm is that it adapts to 
;       the received bit rate. Since the controller uses an internal 
;       RC oscillator which is very sensitive to variations in voltage
;       and temperature, the algorithm must adapt to a frequency 
;       variation between RX and TX modules. 
;
;       One disadvantage is that the AT90LS2343 only contains one 8-bit 
;       timer counter, which reduces the practical variation. 
;       For other applications use the 16-bit counter or an 
;       external XTAL.                
;
;   Data transmission:
;       The internal RC oscillator in each of the modules RX and TX can have up 
;       to +/-20% offset over the entire temperature range. This will give a maximum 
;       clock skew of (1+20%)/(1-20%)= 1,5 or (1-20%)/(1+20%)= 0,6666... = 3/2 or 2/3.
;       The bit stream is Manchester coded.
;
;          |  Get timing   |               |  BYTE 1                       |   BYTE 2  ........        |      STOP     |
;           _   _   _   _   _______                                                                             _______
;       .... \_/ \_/ \_/ \_/       \_______X-x-X-x-X-x-X-x-X-x-X-x-X-x-X-x-X-x-X-x-X-x-........X-x-X-x-X_______/
;          |               |               |
;          |   |   |   |   |   |   |   |   |   |   |   |   |   |   |   |   |   ........|   |   |   |   |   |   |   |   |
;       ............ SYNC  |      SOF      |  DATA ........                                            |      EOP      |
;
;       During sync search, INT0 will be enabled regularly to look for activity on the air
;       The interrupt is never activated in TX mode. Only activated during RX (active period).
;       The trigger edge shall be altered each time this routine completes.
;       In RX data state read last setting and change trigger edge.
;
;       A timer function shall get the timing between each interrupt
;      
;       1. Enter RX_PRECHARGE for 5 bit periods
;       2. Enter RX 
;       3. The receiver shall receive sync for X = (f.eks 8) bit periods before it is allowed to detect SOF
;       4. Wait for SOF
;       5. From now on the receiver will correlate the received bit pattern against timing of sync pattern
;          it will be 4 high and 4 low timing periods, check that none of the values are more than 10%
;          different from the last one.
;          perform simple addition of all periods and left shift by 3 (div 8)    
;
;    SYNC SEARCH:
;       Use constant trigger edge in sync search as this is independent upon phase skew
;       Accept pulses with ~ same length, ..one, two... period offset (quantisation failure).
;       Always save timing of last pulse.
;
;
;***************************************************************************


;***************************************************************************
;
;   EEPROM segment - default system configuration data
;
;       Operation parameters are programmed into the EEPROM of the 
;       micro controller and is used to control various parameters 
;       in the receiver and transmitter algorithms and the protocol
;       in general. A total of 127 bytes can be used.
;
;       - RX sampling period:    RX_SP   10-20kHz 1 byte
;       - Polling period:        POLP         1 s 1 byte
;       - Pre-charge period:     PCP          5 ms 1 byte
;       - Number of sync rep.:   NOPREP  Adjust to total 1.5
;       - Data rate:             DR      1.2 KBPS
;       - Blink sequence:                2 times 0xAB Morse - "victory" 
;
;   System parameters:
;   =================
;       The configuration register values are defined by the 
;       following system parameters (generated by SmartRF Studio):
;       ----------------------------------------------------------
;       X-tall frequency:          12.000000 MHz
;       X-tall accuracy:           +/- 50 PPM
;       RF frequency:              433.920 MHz
;       IF stage:                  200 kHz
;       Frequency separation:      40 kHz
;       Data rate:                 1.2 KBPS
;       Power amplifier class:     Class B
;       RF output power:           0 dBm
;       Mode:                      TX
;       Receiver mode:             Optimum sensitivity
;       LOCK indicator:            Continuous
;
;   Component values, PLL loop filter:
;   ==================================
;       C121 = 33.00 nF
;       C122 = 1500 pF
;       C123 = 330.0 pF
;       R121 = 5.6 k Ohm
;       R122 = 27 k Ohm
;       R123 = 22 k Ohm
;
;   Component values, Input/output match:
;   =====================================
;       C51 = 220 pF
;       C61 = 18 pF
;       L51 = 33 nH
;       L61 = 8.2 nH
;   
; 
;   NOTE: When defining values as words lowest byte is always placed first!
;
;***************************************************************************   
.eseg                                                                       
 
TX:          ; 8x16bit  (Note: only frame 007 is different from RX pre-charge)
    .db 0x08
    .db 0x2A ; Adr:000    Data:082A
    .db 0x23
    .db 0x0B ; Adr:001    Data:030B
    .db 0x40
    .db 0x41 ; Adr:002    Data:0041
    .db 0x7F
    .db 0x9A ; Adr:003    Data:1F9A
    .db 0x8A
    .db 0x29 ; Adr:004    Data:0A29
    .db 0xBA
    .db 0x02 ; Adr:005    Data:1A02
    .db 0xCB
    .db 0x6C ; Adr:006    Data:0B6C
    .db 0xE0
    .db 0x40 ; Adr:007    Data:0040

RX:          ; 1x16bit
    .db 0xE0
    .db 0x40 ; Adr:007   Data:0040 

RX_PC:       ; (Pre-charge mode):  8x16bit    RX pre-charge
    .db 0x00
    .db 0x2A ; Adr:000   Data:002A
    .db 0x23
    .db 0x0B ; Adr:001   Data:230B
    .db 0x40
    .db 0x41 ; Adr:002   Data:0041
    .db 0x7E
    .db 0xE1 ; Adr:003   Data:1EE1
    .db 0x8A
    .db 0x00 ; Adr:004   Data:0A00
    .db 0xBA
    .db 0x02 ; Adr:005   Data:1A02
    .db 0xCB
    .db 0x6C ; Adr:006   Data:0B6C
    .db 0xE2
    .db 0x48 ; Adr:007   Data:0248

PD:          ; Power down, osc. off, 2x16bit 
    .db 0x18
    .db 0x2A ; Adr:000   Data:182A 
    .db 0x40
    .db 0x51 ; Adr:002   Data:0051 
 
POLP:        ; sync search Polling period
    .db 0x00 ;1sec  

PCP:         ; Pre-charge period
    .db 0x00 ; 5ms

RX_SP:       ; RX sampling period
    .db 0x00  

RF_BR:       ; RF bit rate

RF_300_BAUD:
RF_600_BAUD:
RF_1200_BAUD:
RF_2400_BAUD:
RF_4800_BAUD:
RF_9600_BAUD:


PACKET_REQUEST: 
    .db 0x80    
PACKET_ACKNOWLEDGE:
    .db 0x81                   
PACKET_UNKNOWN:
    .db 0xF2     



;**************************************************************************
; Pinout and signal direction of the CC400DB
;**************************************************************************
;
;                AT90LS2343 (internal RC oscillator 1MHz +/- 20%)
;                              _____ _____
;                             |     -     |
;                   RESET*  --| 1       8 |--   VCC
;    CC400(STROBE) <-- PB3  --| 2       7 |--   PB2 SCK (TO)     --> CC400(CLOCK) --[1k]--[push "TX"]--GND
;     CC400(PDATA) <-> PB4  --| 3       6 |--   PB1 MISO (INT0)  --> CC400(DIO)   --[1k]--[push "ON"]--GND
;                      GND  --| 4       5 |--   PB0 MOSI         -->              --[1k]--[<< LED >>]--VCC
;                             |___________|
;
;    NOTE: PCB fix: swap (DIO on PB4 to PB1) and (PDATA on PB1 to PB4). Figure above is according to the new fix        
; 
;**************************************************************************
; PORTB definition                                                         
;************************************************************************** 
    .equ    LED         = 0
    .equ    DIO         = 1
    .equ    CLOCK       = 2
    .equ    STROBE      = 3
    .equ    PDATA       = 4

    .equ    ON_BUTTON   = 1
    .equ    TX_BUTTON   = 2
                                                               
;************************************************************************** 
;    LED    : Light up at power up = output '0'
;    PDATA  : Output '0'
;    CLOCK  : Output '0'
;    STROBE : Output '0'
;    DIO    : Unknown direction from CC400, then select floating input for MCU
;************************************************************************** 
    .equ    PORTB_INIT  = (0<<LED | 0<<PDATA | 0<<CLOCK | 0<<STROBE | 0<<DIO)
    .equ    DDRB_INIT   = (1<<LED | 1<<PDATA | 1<<CLOCK | 1<<STROBE | 0<<DIO)
  
 
;***************************************************************************
;    Register utilization
;***************************************************************************    
    .def    status      = r0    ; This register is used for the high level protocol       
    .def    eep_buffer  = r1    ; Buffer register of EEPROM    
    .def    eep_pt      = r2    ; Pointer in EEPROM segment
    .def    pulse_tmp   = r3    ; Temporary pulse length variable used in correlation alg.         
    .def    pulse_ref   = r4    ; Used as pulse length reference value (The same as last value during sync search)    
    .def    pulse_length = r5   ; This is the counter value/pulse-length from one pos. edge int. to the next pos. edge int. in RX    
    .def    timeout_cnt = r6    ; This is a down counter decrement by one each time entering RX mode, 
                                ; causing the module to sleep after xx sec.                                                                
    .def    rf_sync_cnt = r7    ; In TX mode this is a count down register to send sync pattern and finally SOF
                                ; In RX sync search mode this counter will count down a number of 
                                ; sync pulses before enabling SOF detector.                                
    .def    rf_data_cnt = r8    ; This counter shall count number of bytes sent or received through RF
                                ; This is not cleared during RX_INIT
                                ; incremented during RX_DATA, decremented down to zero during DATA_TO_LED                                                                                         
    .def    x1          = r9    ; Unused
    .def    x2          = r10   ; Unused
    .def    x3          = r11   ; Unused
    .def    x4          = r12   ; Unused
    .def    x5          = r13   ; Unused
    .def    x6          = r14   ; Unused
    .def    x7          = r15   ; Unused    
    .def    x8          = r16   ; Unused
    .def    x9          = r17   ; Unused
    .def    xA          = r18   ; Unused    
    .def    xB          = r19   ; Unused
    .def    xC          = r20   ; Unused
    .def    xD          = r21   ; Unused
    .def    xE          = r22   ; Unused
    .def    tmp         = r23   ; General temporary register               
    .def    state       = r24   ; General state machine in main()                                               
    .def    idle_timer  = r25   ; In IDLE mode value of this timer will compared with preset values 
                                ; change modes (IDLE -> RX_PC -> (RX, ENA, IDLE)-> ...)
                                ; During RX active period, this will be used as a sync_search fail counter                                                                              
    .def    rf_bit_cnt  = r26   ; In TX mode this is a count down register to count the number of bits before 
                                ; reloading rf_buffer from RAM.                                
                                ; In RX mode this is an auto decrement value of pointer to count down the number of bits 
                                ; received before storing rf_buffer into RAM.                  
                                ; This pointer is also used when displaying received data on LED.                                 
    .def    rf_buffer   = r27   ; Temporary register holding rf byte to be transmitted or received. To LED as well.
    .def    rf_ctrl     = r28   ; This ctrl register is a multiple flag register indicating current transmission state etc.                                                                       
    .def    rf_status   = r29   ; This status register is a multiple flag register indicating current transmission state etc.
    .def    data_pt     = r30   ; Note that this register is the ZL register and shall be used to index rf data, name "data_pt" is unused     
    .def    reserved_pt = r31   ; Note that this register is the ZH register and may be used to index rf_data 
                                ; when data area is bigger: This may happen when using other processor
  
;***************************************************************************
;   User Definitions 
;***************************************************************************  

    ;============================================================
    ;= Sync durations                                           =
    ;============================================================
    .equ    TX_SYNC_LENGTH      = 76    ; 76 => ~1.0s @3.0V, 255 => ~1.7sec @5.0V   ; The number of bytes used for sync before sending SOF 
    .equ    RX_SYNC_CNT         = 4     ; After detecting 4 pulses with equal time spacing then we are in sync.
    .equ    RX_NOSYNC_CNT       = 10    ; Give up sync search after RX_NOSYNC_CNT trials to obtain sync.


    ;=====================================================================
    ;= BAUD RATE @5.0V      300     600    1200    2400    4800     9600 =
    ;=====================================================================
    ; RX_TIMER_PRECALE     0x03    0x03    0x03     0x02   0x02     0x01
    ; TX_TIMER_PRESCALE    0x03    0x03    0x03     0x02   0x02     0x01
    ; TX_TIMER_PRESET       224     240     248      192    224      248
    ; 256-TX_TIMER_PRESET:   32      16       8       64     32        8
    ; TCNT0 (max/min):
    
    
    ;=====================================================================
    ;= BAUD RATE @3.0V      300     600    1200    2400    4800     9600 =
    ;=====================================================================
    ; RX_TIMER_PRECALE     0x04    0x03    0x02     0x03   0x02     0x02
    ; TX_TIMER_PRESCALE    0x04    0x03    0x02     0x03   0x02     0x02
    ; TX_TIMER_PRESET       232     208     235      244    208      232
    ; 256-TX_TIMER_PRESET:   24      48      21      12     48       24
    ; TCNT0 (max/min):       NO      NO      NO      NO    NO      NO
 
     
    .equ    RX_TIMER_PRESCALE   = 0x03  ; Prescaler during RX DATA reception.
    .equ    TX_TIMER_PRESCALE   = 0x03  ; Prescaler during TX DATA transmission. Set precaler to 0x03 = div64
    .equ    TX_TIMER_PRESET     = 248   ; prescaler = 0x03 = XTAL / 64: Count up 8: 256 - 8 = 248     
                                                                 
                                                                  
    ;============================================================
    ;= Sample presition                                         =
    ;============================================================    
    .equ    PULSE_OFFSET        = 5     ; 3-depending on duty-cycle, 4-OK, 5-OK, 6-TJA, Edge presition: minimum number is 2 = +/-1 LSB, 3 = +/-2 LSB....                                 
    ;.equ    SOF_OFFSET          = 3    ; Maximum allowed offset in neg edge of SOF. Same rules as above.       
    ;.equ    DATA_OFFSET         = 3    ; Now looping a number of times before giving up (3 works fine)
                                            
                            
    ;============================================================                                                     
    ;= Maximum and minimum CHIP lengths in sync pattern         =   ;SOF pulse may now be 4*12 = 48 in length
    ;============================================================ 
    ;.equ    RX_TIMER_SYNC_MAX    = 12  ; Shall be greater than (256 - TX_TIMER_PRESET) * (1,5) = 12, must not be grater than 256/4,25 = 60 
    .equ    RX_TIMER_SYNC_MIN    = 7    ; Shall be less than ((256 - TX_TIMER_PRESET) * 0,66) - PULSE_OFFSET + 1 = 5
    
    
    ;============================================================  
    ;= Definitions of IDLE periode (CC400DB will be in IDLE most of the time)
    ;============================================================ 
    ; - After RX, enter IDLE mode
    ; - Process state machine in main() and wait until POWERDOWN_TIMEOUT has reached IDLE_TIMER_SET_RX_PC (down counter)
    ; - Continue count down until zero and enter RX mode. Look for sync (and get data) At value = zero: 
    ;     1. decrement timeout counter, if zero then enter power down
    ;     2. Set CC400 to RX (SYNC SEARCH), redefine timer parameters
    ; - Preset idle counter (idle_timer) to IDLE_TIMER_PRESET

    .equ    POWERDOWN_TIMEOUT    = 120  ; Do the RX polling 120 times then power down (120 sec = 2 min ), max 255
    .equ    IDLE_TIMER_PRESCALE  = 0x02 ; Set precaler to 0x02 = div8
    .equ    IDLE_TIMER_PRESET    = 200  ; Count down from this value
    .equ    IDLE_TIMER_SET_RX_PC = 2    ; ~10ms Count down match to enter RX pre-charge

                                                                     
    ;==========================    
    ;= SHOWING RX DATA ON LED =
    ;==========================
    .equ    LED_DATA_TIMER_PRESCALE = 0x05  ; div 1024 
    .equ    LED_DATA_TIMER_PRESET   =  206  ; 206 @3.0V = ~125ms pr. bit = 1 sec. pr. byte



;*************************************************************************** 
; FLAG definitions of status register                                        
;***************************************************************************  
    .equ    DATA_FLAG   = 0  ; This flag will be set high when timer shall show data information on LED  
    .equ    RF_ACK_FLAG = 1  ; This flag will be set high when receiving ACK from slave
 
     
;*************************************************************************** 
; FLAG definitions of rf_status register                                        
;***************************************************************************  
    .equ    SYNC_FLAG   = 0  ; This flag is set when the receiver has detected or is sending SYNC 
    .equ    FRM_FLAG    = 1  ; This flag will be set high when the sending frame information: SOF and EOP
    .equ    SOF_FLAG    = 2  ; This flag is set when the receiver has received or sent Start Of Frame
    .equ    EOP_FLAG    = 3  ; This flag is set when the receiver has received or sent End Of Package    
    .equ    RX_FLAG     = 4  ; This flag will be set when in active RX (sync search or data read)
    .equ    RD_FLAG     = 5  ; This flag will be set when receiving data
    .equ    RDR_FLAG    = 6  ; This flag is set when (RF) Receive Data is Ready (Set when SYNC + SOF + EOP detected)    
    .equ    CHIP_FLAG   = 7  ; This flag is holding the next bit to be transmitted as a pipeline
     
    .equ    STATUS_INIT = (0<<FRM_FLAG|0<<SYNC_FLAG|0<<1<<SOF_FLAG|0<<EOP_FLAG|0<<RDR_FLAG|0<<RD_FLAG|0<<RX_FLAG|0<<CHIP_FLAG))

;*************************************************************************** 
; FLAG definitions of rf_ctrl register                                        
;***************************************************************************  
    .equ    TD_FLAG     = 5  ; This flag shall be set before Transmitting Data (select TX routine in timer interrupt), flag is reset when TX Complete
    .equ    TPOL_FLAG   = 6  ; This flag indicates polarity of the transmit bit, Invert if TPOL_FLAG = '1'                             
    .equ    RPOL_FLAG   = 7  ; This flag indicates polarity of the transmit bit, Invert if TPOL_FLAG = '1'

    
;*************************************************************************** 
; General definitions
;***************************************************************************  
    .equ    LSB         = 0             ; Most Significant Bit
    .equ    MSB         = 7             ; Least Significant Bit


;***************************************************************************
;    RAM utilisation address $60 - $DF (128 bytes)
;*************************************************************************** 
    .equ    STACK_POINTER         = 0x9F    ;       Program stack:  0x60 - 0x9E   64 bytes (Growing down)
    .equ    RF_DATA_BUFFER        = 0xA0    ;       RF Data buffer: 0xA0 - 0xDF   64 bytes (Growing up)
    .equ    RF_DATA_BUFFER_SIZE   = 64
    ;.equ    DATA_STACK           = 0xD0    ;       Data stack:     0xD0 - 0xDF   16 bytes (Unused)     
   
    
;*************************************************************************** 
; State machine constants in main()
;***************************************************************************      
    .equ    IDLE                  = 1        
    .equ    SHOW_RX_DATA          = 2    
    .equ    RF_ACKNOWLEDGE        = 3
    .equ    RF_REQUEST            = 4
    .equ    WAIT_FOR_TX_RELEASE   = 5            
    .equ    WAIT_FOR_ON_RELEASE   = 6              
  

;***************************************************************************
;
;    Macro's
;
;*************************************************************************** 
.macro LED_ON
;    sbi     DDRB, LED                  ; Set port to output
    cbi     PORTB, LED                  ; Turn on LED
.endmacro                   

.macro LED_OFF            
    sbi     PORTB, LED                  ; Turn off LED
;    cbi     DDRB, LED                  ; Make input with pullup
.endmacro                   

.macro CFG_TX_BUTTON
    ; Configure button-I/O as inputs with pullups 
    cbi     DDRB, TX_BUTTON     
    sbi     PORTB, TX_BUTTON
.endmacro
 
.macro CFG_ON_BUTTON
    ; Configure button-I/O as inputs with pullups 
    cbi     DDRB, ON_BUTTON 
    sbi     PORTB, ON_BUTTON
.endmacro


 
;***************************************************************************
;
;    Vector definitions
;
;*************************************************************************** 
    .cseg
    .org    $0000                       ; Interrupt vectors

    rjmp    RESET                       ; Reset handler
    rjmp    EXT_INT0                    ; IRQ0 handler
    rjmp    TMR0_OVF                    ; Timer0 overflow handler



;***************************************************************************
;
;    Resolving reset condition
;
;***************************************************************************
; 
; The AT90S2343 provides three sources of reset:
; =============================================
;
;    * Power-On Reset. The MCU is reset when the supply
;      voltage is below the power-on reset threshold (V POT ).
;    * External Reset. The MCU is reset when a low level is
;      present on the RESET pin for more than 50 ns.
;    * Watchdog Reset. The MCU is reset when the Watchdog
;      timer period expires and the Watchdog is enabled.
;
;***************************************************************************
RESET:   
    ; The MCUSR register provides information on which reset source caused MCU reset          
    
    in       tmp, MCUSR                 ; Must read High IO address with IN instruction, SBIC cant handle above $1F
    sbrc     tmp, PORF                  ; Checking the Power On Reset Flag
    rjmp     COLD_INIT                  ; Now there was a power-on reset                        
    sbrc     tmp, EXTRF                 ; Checking the External Reset Flag
    rjmp     HOT_RESET                  ; Now there was an external reset
    rjmp     WATCHDOG_RESET             ; Now there was a watchdog reset    



;***************************************************************************
;
;    Special setup for HOT_RESET
;
;***************************************************************************
HOT_RESET:    
    ;cbi     MCUSR, EXTRF               ; Clearing the bits of MCUSR in order to detect the next reset condition     
    rjmp  COLD_INIT



;***************************************************************************
;
;    Special setup for WATCHDOG_RESET
;
;***************************************************************************
WATCHDOG_RESET:                        
    rjmp  COLD_INIT


               
;***************************************************************************
;
;    Special setup for POWER-ON_RESET
;
;***************************************************************************
COLD_INIT:                       
    clr     tmp
    out     MCUSR, tmp                  ; Clearing the bits of MCUSR in order to detect the next reset condition

    
                                                                                                                                                
;***************************************************************************
;
;    MCU Initialization
;
;***************************************************************************
INIT:
   
INIT_STACK_POINTER:
    ldi     tmp, STACK_POINTER
    out     SPL, tmp

INIT_PORTS:
    ldi    tmp, PORTB_INIT
    out    PORTB, tmp           
    ldi    tmp, DDRB_INIT
    out    DDRB, tmp                                                                                                                   
    
INIT_REGISTERS:
    rcall   CLEAR_ALL_REGISTERS   
    clr     rf_status                   ; Same as ldi rf_status, STATUS_INIT
    clr     rf_ctrl    
    ldi     tmp, POWERDOWN_TIMEOUT
    mov     timeout_cnt, tmp
    clr     idle_timer    
    clr     status 
                             
CHECK_BUTTON_DEFAULT:
    CFG_TX_BUTTON
    CFG_ON_BUTTON

INIT_WATCHDOG:
    ldi    tmp, 0b00001111              ; Timeout after 2048k cycles = ~2 sec
    out    WDTCR, tmp                   ; Watchdog every mS 
 
INIT_TIMER:
    ldi    tmp, IDLE_TIMER_PRESCALE 
    out    TCCR0, tmp                   ; Incrementing every 64 us overflow every 2 mS     
         
INIT_INTERRUPT:
    ldi     tmp, 0b00000010      
    out     TIMSK, tmp                  ; Enable timer overflow interrupt

    ldi     state, WAIT_FOR_ON_RELEASE          
    
    ;rcall   SETUP_CC400_PD
     
    sei                                 ; Enable global interrupt
      

                                                   
;***************************************************************************
;
;    Main program loop
;
;***************************************************************************
MAIN:                     
    ;mov     tmp, state
    ;rcall   print_tmp_value

    ; =================
    ; = switch(state) =
    ; =================
    cpi     state, IDLE                 ; Wait for new RX data or TX button down
    breq    STATE_IDLE      
    cpi     state, SHOW_RX_DATA
    breq    STATE_SHOW_RX_DATA         
    cpi     state, RF_ACKNOWLEDGE
    breq    STATE_RF_ACKNOWLEDGE        
    cpi     state, RF_REQUEST
    breq    STATE_RF_REQUEST    
    cpi     state, WAIT_FOR_TX_RELEASE
    breq    STATE_WAIT_FOR_TX_RELEASE             
    cpi     state, WAIT_FOR_ON_RELEASE
    breq    STATE_WAIT_FOR_ON_RELEASE             
    rjmp    STATE_IDLE                  ; Default will be IDLE
    
                    
    ; =================
    ; = case ........ =
    ; =================                 
STATE_IDLE:                                                                              
;----------
    sbrc    rf_ctrl, TD_FLAG            ; Wait until current transmission is finished
    rjmp    END_STATE

    sbrs    rf_status, RDR_FLAG         ; First check RDR bit, if no new data then
    rjmp    CHECK_TX_BUTTON             ; jump...    

FORMAT_NEW_RX_DATA:    
    rcall   RX_DECODE                   ; else decode data in RX buffer and Set DATA_FLAG if it shall be displayed   
    ldi     state, SHOW_RX_DATA
    rjmp    END_STATE                                           

CHECK_TX_BUTTON:    
    sbis    PINB, TX_BUTTON             ; If the TX button pushed, then 
    ldi     state, RF_REQUEST           ; Change state
    rjmp    END_STATE

                      
STATE_SHOW_RX_DATA:              
;------------------
    sbrc    status, DATA_FLAG           ; Wait until DATA_FLAG is cleared an msg to led is finished
    rjmp    END_STATE                   ; Break;                                                       
                                                                                 
    sbrc    status, RF_ACK_FLAG         ; Skip if TD flag cleared. Then we are in RX mode    
    rjmp    REPLY_MSG
    ldi     state, IDLE    
    rjmp    END_STATE                   ; Break;    
REPLY_MSG:                       
    ldi     state, RF_ACKNOWLEDGE           
    rjmp    END_STATE                   ; Break;         

 
STATE_RF_ACKNOWLEDGE:
;--------------------        
    ; Load device address into TX buffer
    cli                                 ; Disable all interrupts = disable RX, don't want to receive any data now
    rcall   ACK_ENCODE                  ; Define SRAM buffer etc.
    
    rcall   SETUP_CC400_TX              ; Set CC400 to TX with parameters defined in EEPROM
    rcall   TX_INIT                     ; Set TX variables, timer, sync length etc.
    ldi     state, IDLE
    sei                                 ; Enable interrupts again    
    rjmp    END_STATE                   ; Break;

                                                                                                                                              
STATE_RF_REQUEST:    
;----------------        
    ; Load device address into TX buffer
    cli                                 ; Disable all interrupts = disable RX, don't want to receive any data now
    rcall   TX_ENCODE                   ; Define SRAM buffer etc.
    
    rcall   SETUP_CC400_TX              ; Set CC400 to TX with parameters defined in EEPROM
    rcall   TX_INIT                     ; Set TX variables, timer, sync length etc.
    ldi     state, WAIT_FOR_TX_RELEASE
    sei                                 ; Enable interrupts again    
    rjmp    END_STATE                   ; Break;


STATE_WAIT_FOR_TX_RELEASE:                                                
;-------------------------
    sbic    PINB, TX_BUTTON
    ldi     state, IDLE
    rjmp    END_STATE                   ; Break;


STATE_WAIT_FOR_ON_RELEASE:
;-------------------------        
    sbic    PINB, ON_BUTTON
    ldi     state, WAIT_FOR_TX_RELEASE 
    rjmp    END_STATE                   ; Break;

END_STATE:


RESET_WATCHDOG:         
    wdr     ; kick the dog, this instruction shall not be used any other places as this will decrease watchdog security level of software failure.

    ; If timeout then enter sleep mode                                                                        
    ; enter sleep and wait for external level interrupt from "ON" button
    ; (timer interrupt will prevent watchdog reset)
    ; ======================================================================  
    cli 
    in      tmp, MCUCR                  ; Read MCUCR settings (MCU CONTROL REGISTER)
    cbr     tmp, 1<<SM                  ; Set Sleep Enable bit
    sbr     tmp, 1<<SE                  ; Reset Sleep Mode bit (select idle)           
    out     MCUCR, tmp                  ; Write back
    sei
    sleep                               ; Put MCU in IDLE mode              
    rjmp    MAIN
 

      
;***************************************************************************
;
;    TX Encode, Set parameters according to contents of SRAM
;
;***************************************************************************
TX_ENCODE:                 
    cli
                 
    ; Setup pointer to first byte in SRAM (data buffer)
    ldi     ZL, RF_DATA_BUFFER          ; RF_DATA_BUFFER is the first available address
                                    
    ldi     tmp, PACKET_REQUEST    
    mov     eep_pt, tmp
    rcall   eeget
    st      Z+, eep_buffer
    
    ; User data
    ldi     tmp, 0b10101011
    st      Z+, tmp
    ldi     tmp, 0b10000001
    st      Z+, tmp            
    ldi     tmp, 0b01010111
    st      Z, tmp          
    
    ldi     tmp, 4                      ; Setup number of bytes to be transmitted                      
    mov     rf_data_cnt, tmp
    ldi     ZL, RF_DATA_BUFFER          ; RF_DATA_BUFFER is the first available address             
    sei
    ret
 


;***************************************************************************
;
;    TX Encode, Set parameters according to contents of SRAM
;
;***************************************************************************
ACK_ENCODE:             
    cli         
    
    ldi     tmp, 1<<RF_ACK_FLAG       
    com     tmp
    and     status, tmp 
       
         
    ; Setup pointer to first byte in SRAM (data buffer)
    ldi     ZL, RF_DATA_BUFFER          ; RF_DATA_BUFFER is the first available address
    ld      rf_data_cnt, Z              ; Restore data count for acknowledge
                                       
    ; User data replace first byte of message       
    ldi     tmp, PACKET_ACKNOWLEDGE
    mov     eep_pt, tmp
    rcall   eeget
    st      Z, eep_buffer

    sei                  
    ret
 


;***************************************************************************
;
;    RX Decode
;    
;    This routine shall check if there has been received any new data on RF 
;    which means that if SYNC, SOF and EOP was found, then RDR has been set
;    to '1'. In this case decode data in RAM buffer.
;                                                                            
;    The data buffer in RAM may be of any dimension but should be limited to 
;    64 bytes. 
;
;***************************************************************************
RX_DECODE:              
    cli                                 ; Disable interrupts so they don't fuck up tmp variable               
    cbr     rf_status, 1<<RDR_FLAG      ; Incoming data has now been processed
                                 
    tst     rf_data_cnt                 ; Don't show anything on the display is RX data count is less than 1
    breq    END_RX_DECODE                                                                          
           
    ldi     tmp, 1             
    cp      rf_data_cnt, tmp            ; Don't show anything on the display is RX data count is only packet header
    breq    END_RX_DECODE                                                                          
                                                                                  
    ldi     ZL, RF_DATA_BUFFER          ; Setup pointer to first byte in SRAM (data buffer)
    ld      rf_buffer, Z                ; Get data from first location in data buffer

    ldi     tmp, PACKET_REQUEST
    mov     eep_pt, tmp
    rcall   eeget                       ; Get data value of PACKET_REQUEST definition
                                           
    cp      eep_buffer, rf_buffer       ; Compare packet header with data value of PACKET_REQUEST
    breq    RX_REQUEST                  ; This was a request so jump
                                                     
    ldi     tmp, PACKET_ACKNOWLEDGE
    mov     eep_pt, tmp
    rcall   eeget                       ; Get data value of PACKET_REQUEST definition
                                           
    cp      eep_buffer, rf_buffer       ; Compare packet header with data value of PACKET_REQUEST
    breq    RX_ACKNOWLEDGE              ; This was a request so jump
    
                          
RX_UNKNOWN:                          
    ldi     ZL, RF_DATA_BUFFER          ; Setup pointer to first byte in SRAM (data buffer)
                                        
    ldi     tmp, PACKET_UNKNOWN
    mov     eep_pt, tmp
    rcall   eeget
    mov     rf_buffer, eep_buffer
    
    ldi     tmp, 1                      ; Setup number of bytes to be transmitted            
    mov     rf_data_cnt, tmp          

    rjmp    END_RX_DECODE                                        
                                                     
RX_ACKNOWLEDGE:                      
    ldi     ZL, RF_DATA_BUFFER          ; Setup pointer to first byte in SRAM (data buffer)
    st      Z+, rf_data_cnt             ; Store number of received bytes into first location and increment pointer
    dec     rf_data_cnt                 ; Reduce data count by one as this will be used in the DATA_TO_LED routine
            
    ld      rf_buffer, Z+               ; Load second byte from SRAM and point at next location 
    ldi     rf_bit_cnt, 0x80            ; Point at first bit in rf_buffer

    ldi     tmp, 1<<DATA_FLAG           ; Set a flag to indicate that that new data has arrived
    or      status, tmp            
    rjmp    END_RX_DECODE                                        
                                                                                                                                   
RX_REQUEST:                                  
    ldi     ZL, RF_DATA_BUFFER          ; Setup pointer to first byte in SRAM (data buffer)
    st      Z+, rf_data_cnt             ; Store number of received bytes into first location and increment pointer
    dec     rf_data_cnt                 ; Reduce data count by one as this will be used in the DATA_TO_LED routine
            
    ld      rf_buffer, Z+               ; Load second byte from SRAM and point at next location 
    ldi     rf_bit_cnt, 0x80            ; Point at first bit in rf_buffer

    ldi     tmp, 1<<DATA_FLAG           ; Set a flag to indicate that that new data has arrived
    or      status, tmp
            
    ldi     tmp, 1<<RF_ACK_FLAG         ; Now we have received a request, Set a flag to ACK
    or      status, tmp     
    
    ;rjmp    END_RX_DECODE                                                                                                         
                                                                                                                                                                           
END_RX_DECODE:                          
    sei                                 ; Enable interrupts again
    ret 



;***************************************************************************
;
;    TX Init
;
;       This procedure shall set up timer and flags according to tx mode
;
;***************************************************************************
TX_INIT:       
    cli     
    ldi     tmp, POWERDOWN_TIMEOUT
    mov     timeout_cnt, tmp
                        
    ser     rf_buffer                   ; Sync pattern 0xFF will generate: [10101010 10101010] - [1010101....       
                                    
    ; Setup How long time (number of bytes) to send IDLE before transmitting SOF
    
    ldi     tmp, TX_SYNC_LENGTH         ; rf_sync_cnt always >= 0x01        
    mov     rf_sync_cnt, tmp
    ldi     rf_bit_cnt, 16              ; 16 CHIPS 
   
    cbr     rf_status, (1<<FRM_FLAG | 1<<SOF_FLAG | 1<<EOP_FLAG | 1<<RDR_FLAG | 1<<CHIP_FLAG)
    sbr     rf_status, (1<<SYNC_FLAG) 
    sbr     rf_ctrl, (1<<TD_FLAG | 1<<TPOL_FLAG) 
    
    ldi     tmp, TX_TIMER_PRESCALE      ; Set timer to correct baud rate 
    out     TCCR0, tmp                  ; Incrementing every 64 us @osc. = 1Mhz   
 
    ldi     tmp, 0b00000010      
    out     TIMSK, tmp                  ; Enable timer overflow interrupt (just in case)
    
    ldi     tmp, TX_TIMER_PRESET        
    out     TCNT0, tmp                  ; Set delay to first interrupt, overflow = CHIP rate
    sbi     DDRB, DIO                   ; Set DIO pin to output just in case
    
    ldi     tmp, 1<<TOV0
    out     TIFR, tmp                   ; Clear timer interrupt flag if this has been set
    sei 
    ret    



;***************************************************************************
;
;    RX Init
;
;       Common routine to clean up and re-enter idle
;       NOTE: This is not an ordinary sub-routine. It shall be entered
;       via a "rjmp RX_INIT" command and only be executed from within an
;       interrupt as it exits with "reti". 
;
;***************************************************************************
RX_INIT:        
    cbi     DDRB, DIO                   ; Set to input
    sbi     PORTB, DIO                  ; pullup            
    
    LED_OFF                             ; Just in case                 
    
    clr     tmp                         ; Disable ext_int0
    out     GIMSK, tmp                  ; INT0 is the only bit in GIMSK for AT90S2343 take care when using other MCU's    
    
    ; Flush any pending interrupt     
    ldi     tmp, 1<<INT0
    out     GIFR, tmp             
         
    ; No longer in Tx mode    
    cbr     rf_status, 1<<RD_FLAG | 1<<EOP_FLAG | 1<<FRM_FLAG | 1<<SOF_FLAG | 1<<SYNC_FLAG| 1<<CHIP_FLAG | 1<<RX_FLAG     
    cbr     rf_ctrl, 1<<TD_FLAG    
           
    ldi     idle_timer, IDLE_TIMER_PRESET      
    
    rcall   SETUP_CC400_IDLE                 
    
    ldi     tmp, IDLE_TIMER_PRESCALE    ; Set timer prescaler back to RX timing
    out     TCCR0, tmp                  ; Incrementing every 64 us overflow every 2 mS @ 1MHz             
    
    ; Setup pointer to first byte in SRAM (data buffer)
    ldi     ZL, RF_DATA_BUFFER          ; RF_DATA_BUFFER is the first available address
    
    reti


  
;***************************************************************************
;
;    8 bit timer/counter Interrupt routine
;                   
;    Description:    
;       There are in general 2 modes of operation: TX and RX.         
; 
;       In TX mode the timer is used for data transmission.
;       In RX active mode the timer is used for bit separation timing
;       and in RX polled mode for IDLE timing. The timer shall periodically wake 
;       up the controller and perform a short RX "listen" for RF traffic.
;
;       TX mode: run at CHIP rate = 2*bitrate 
;       timer_divison and timer_reload are fetched from EEPROM to register
;       and reloaded.                                                         
; 
;       When in TX mode check the SYNC down counter and send sync until counter = 0
;       then send SOF (Start Of Frame) followed by data from RF_DATA_BUFFER in SRAM.                 
; 
;       The RX mode has two states. These are active mode (reading data) and sync search.
;       In sync search mode, the controller will wake up once each second to
;       check if there are any activity on the air.
;
;       Power down is not considered a mode of operation. After a timeout
;       from the RX polled mode and no detection of RF signal, the controller shall 
;       enter power down mode. A low level interrupt will force the 
;       program counter to execute from reset vector. After reset the 
;       controller will execute init and set the module into RX polled mode
;       (sync search). 
;
;***************************************************************************
TMR0_OVF:            
    ;First determine the mode: RX (most critical), then TX (no jitter)
    ;========================    
    sbrc    rf_status, RX_FLAG          ; Check if timer is in the active RX mode
    rjmp    RX_TMR0_OVERFLOW            ; If set then go and increment the CHIP counter high byte
    
    sbrc    rf_ctrl, TD_FLAG            ; Skip if TD flag cleared. Then we are in RX mode
    rjmp    TX_TMR0_OVF    
    
    sbrc    status, DATA_FLAG           ; Here we can show bits on the led
    rjmp    DATA_TO_LED    
                                                                                                                       
;===========
;  IDLE mode
;===========                              
IDLE_TMR0_OVF:                                                                                                                                                                                                                 
    dec     idle_timer                  ; Decrement the idle timer counter
    breq    RX_TIMER_ZERO               ; If zero then start RX period
    cpi     idle_timer, IDLE_TIMER_SET_RX_PC
    breq    SET_RX_PC                   ; If equal to IDLE_TIMER_SET_RX_PC then enter precharge mode
    reti
    
SET_RX_PC:
    rcall   SETUP_CC400_RX_PC           ; Configure CC400 to RX_PRECHARGE mode
    LED_ON                              ; Turn ON hart pulse during RX_PC
    reti 
                                                   
RX_TIMER_ZERO:     
    dec     timeout_cnt    
    brne    SET_RX
    rcall   power_down                  ; Timeout -> enter power down and wait for anyone to push "ON" button        
    
SET_RX:                                                                            
    LED_OFF                             ; Turn OFF hart pulse
    ldi     tmp, RX_TIMER_PRESCALE          
    out     TCCR0, tmp                  ; Set precaler to correct baud rate 

    sbr     rf_status, 1<<RX_FLAG           
    cbr     rf_status, 1<<SOF_FLAG
    
    cbi     DDRB, DIO                   ; Make data DIO pin input
    sbi     PORTB, DIO                  ; Turn on pull-up
    
    rcall   SETUP_CC400_RX              ; Configure CC400 to RX mode
         
    ldi     tmp, 1<<ISC01 | 1<<ISC00
    out     MCUCR, tmp                  ; Set external interrupt trigger to rising edge
    
    ldi     tmp, 1<<INT0
    out     GIMSK, tmp                  ; Enable external int. This is the only bit in GIMSK for AT90S2343

    clr     pulse_length                ; Clear all variables used for pulse length measurements
    clr     pulse_ref                         
     
    clr     rf_data_cnt                 ; Reset the RF RX byte counter.
    
    ldi     idle_timer, RX_NOSYNC_CNT   ; Preset number of errors in sync before giving up RX                               
     
    ldi     tmp, RX_SYNC_CNT            ; Preset number of sync pulses to detect before enter definition of "IN-SYNC"
    mov     rf_sync_cnt, tmp
    reti                           

 

;***************************************************************************
;
;    Procedure:         DATA_TO_LED
;    
;    Description:       Write one bit (same as T in SREG) into rf_buffer[rf_bit_cnt]
;                       and store in RAM if rf_buffer full
;
;    Variables affected: rf_buffer, rf_bit_cnt, RAM
;
;    First send data with current values, then update buffer and pointers
;
;***************************************************************************              
DATA_TO_LED:                
    ldi     tmp, POWERDOWN_TIMEOUT
    mov     timeout_cnt, tmp     
    tst     rf_data_cnt
    brne    UPDATE_LED     
EXIT_DATA_TO_LED:     
    ldi     tmp, 1<<DATA_FLAG           ; Clear the DATA to led flag
    com     tmp
    and     status, tmp
    rjmp    RX_INIT                     ; END OF DATA                
UPDATE_LED:
     

    mov     tmp, rf_buffer              ; Get bit from location
    and     tmp, rf_bit_cnt
    breq    LED_LOW_BIT
LED_HIGH_BIT:
    LED_ON            
    rjmp    END_LED_BIT        
LED_LOW_BIT:    
    LED_OFF
END_LED_BIT:           
                                                                                 
    ;                                                                                 _____                                                                              
    ; For future external interface make a clock pulse on SCK active low pulse  _____/     \______.....
    ; => sbi     PORTB, CLOCK           ; Make clock pull-up or high level           |  1  |  2  |  <- DATA_TO_LED time frame
    ; => sbi     DDRB, CLOCK            ; Clock is now active high output            
           
    lsr     rf_bit_cnt                  ; Shift bit pointer one position to the right         
    brne    END_SRAM_GET_BIT            ; Don't load new value into rf_buffer
    dec     rf_data_cnt                 ; Check if this was the last byte 
       
    brne    RELOAD_FROM_SRAM            ; No then reload 
    rjmp    END_SRAM_GET_BIT            ; Now Set LED duration for the last time
    
RELOAD_FROM_SRAM:      
                                        ; First check if Z pointer within range                         
    cpi     ZL, (RF_DATA_BUFFER + RF_DATA_BUFFER_SIZE)
    brlo    GET_SRAM_DATA
    ldi     ZL, RF_DATA_BUFFER          ; Reset pointer (wrap around in data buffer)
GET_SRAM_DATA:      
    ld      rf_buffer, Z+
    ldi     rf_bit_cnt, 0x80            ; Reset pointer    

END_SRAM_GET_BIT:
    ldi    tmp, LED_DATA_TIMER_PRESCALE ; Set timer prescaler
    out    TCCR0, tmp                   ; Incrementing every 64 us overflow every 2 mS     

    ldi    tmp, LED_DATA_TIMER_PRESET   ; Set timer counter value    
    out    TCNT0, tmp                   ; Incrementing every 64 us overflow every 2 mS         
    reti
    
TURN_OFF_LED_CLOCK:                     ; Set both data and clock as inputs in case these shall be used for other purposes as well    
    cbi     PORTB, CLOCK                ; Set high (high fanout)
    cbr     rf_status, CHIP_FLAG
    rjmp    END_SRAM_GET_BIT


;===========
;  RX mode
;===========                              
RX_TMR0_OVERFLOW:                       ; Sync search timeout
    rjmp    RX_INIT                     ; Now there has been a wrap around in the pulse counter - Too long time without activity


;===========
;  TX mode
;===========                                            
TX_TMR0_OVF:      

    ;**********************************************************************
    ; Update I/O - time critical (requires no jitter) with value from last prosessing
    ;**********************************************************************   
    ldi     tmp, TX_TIMER_PRESET
    out     TCNT0, tmp                  ; Set time to next interrupt
    sbi     DDRB, DIO                   ; Set DIO pin to output just in case
   
    ;**********************************************************************
    ; Set next value in pipelined outbut buffer  (constant cycle, no jitter)
    ;**********************************************************************
    mov     tmp, rf_buffer              ; Copy rf_buffer to tmp
    sbrc    rf_ctrl, TPOL_FLAG          ; Invert the entire tmp register if TPOL = '1'
    com     tmp                                                      
    bst     tmp, CHIP_FLAG              ; Store CHIP from tmp register to T                                                                                                                                                                
    in      tmp, PORTB                  ; Read PORTB settings                    
    bld     tmp, DIO                    ; Bit load from T to DIO pin                   
    out     PORTB, tmp                  ; Update PORTB with new DIO setting        
 
    ;**********************************************************************
    ; Store next bit, then toggle MSB in rf_buffer
    ;**********************************************************************
    bst     rf_buffer, MSB              ; D7 store from rf_buffer register to T
    bld     rf_status, CHIP_FLAG        ; Bit load from T to rf_status register (This is the next bit to be transmitted) 
       
    ldi     tmp, 1<<MSB                 ; TREG FLAG must always be D7             
    eor     rf_buffer, tmp              ; Change
        
    ;**********************************************************************
    ; Shift register left if 16 chips (8 bits) transmitted
    ;**********************************************************************
    dec     rf_bit_cnt    
    sbrc    rf_status, FRM_FLAG         ; If frame info: SOF or EOP then shift every time
    rjmp    SHIFT_BIT        
    sbrc    rf_bit_cnt, LSB             ; Else check LSB of counter and shift only each second time
    reti
                 
SHIFT_BIT:        
    lsl     rf_buffer                   ; Shift contents of rf_buffer left (remove MSB)
    tst     rf_bit_cnt
    breq    RF_BUFFER_RELOAD            ; Yes, Buffer register empty, then reload it
    reti                                ; No, Buffer register not empty, don't update yet                         
                                                      
RF_BUFFER_RELOAD:                 
    ;**********************************************************************
    ; Reload buffer when rf_bit_cnt = zero (and change state)
    ;**********************************************************************                      
    sbrc    rf_status, SYNC_FLAG
    rjmp    TX_SYNC
                                                    
    sbrc    rf_status, SOF_FLAG
    rjmp    TX_DATA_INIT         
    
    sbrc    rf_status, EOP_FLAG
    rjmp    RX_INIT         
    rjmp    LOAD_FROM_SRAM 
 
TX_SYNC:                              
    ser     rf_buffer                   ; Reload sync pattern 0xFF into rf_buffer       
    ldi     rf_bit_cnt, 16                                      
    dec     rf_sync_cnt                 ; Decrement sync pattern down counter.
    breq    TX_SOF_INIT                 ; Check rf_sync_cnt register. If not zero then send sync and dec rf_sync_cnt. 
                                        ; Else send SOF
    reti

TX_SOF_INIT:    
    cbr     rf_status, 1<<SYNC_FLAG     ; No longer sending sync                      
    sbr     rf_status, 1<<SOF_FLAG | 1<<FRM_FLAG
    ldi     rf_buffer, 0xF0
    ldi     rf_bit_cnt, 8                                       
    reti
        
TX_DATA_INIT:
    cbr   rf_status, 1<<FRM_FLAG | 1<<SOF_FLAG     ; Clear this flag to enable mancester encoding                       
LOAD_FROM_SRAM:        
    tst     rf_data_cnt                 ; Decrement the buffer byte counter
    breq    TX_EOP_INIT                 ; When data counter reached zero then initialize EOP
    ld      rf_buffer, Z+               ; Z pointer is rf_buffer pointer, load from SRAM to register and post inc
    ldi     rf_bit_cnt, 16
    dec     rf_data_cnt
    reti
    
TX_EOP_INIT:    
    sbr     rf_status, 1<<EOP_FLAG | 1<<FRM_FLAG     ; This routine will only be initiated once before exit    
    ldi     rf_buffer, 0x0F
    ldi     rf_bit_cnt, 10              ; 8+1+1 (1 level pipelining + "zero bit")                                        
    reti

 

;***************************************************************************
;
;    External interrupt INT0
;
;    This interrupt service routine will be activated when the selected positive
;    or negative edge (as selected) occur.
;
;***************************************************************************
;    Wakeup is not considered a part of this routine as it will always 
;    execute from RESET vector and never generate INT0 interrupt.
;***************************************************************************      
EXT_INT0:        ; IRQ0 interrupt routine          
         
    ;**********************************************************************
    ; First get timer value.
    ;**********************************************************************          
    in      pulse_length, TCNT0         ; Get chip timing (low byte from HW timer)           
    in      tmp, MCUCR                  ; Read MCU control register
    sbrc    tmp, ISC00                  ; Check level of last interrupt, skip if neg. edge, ISC00 = '1' = POS_EDGE    
    rjmp    POS_EDGE_INT

    ;**********************************************************************
    ; This shall be a continuos check for SOF
    ;**********************************************************************                                         
NEG_EDGE_INT:                                                                   
    mov     pulse_tmp, pulse_length     ; Get ref value and divide by 2. If equal then this is the SOF   
    lsr     pulse_tmp                   ; Divide by 2
    rjmp    COMP_PULSE
                 
    ;**********************************************************************
    ; Determine mode RX state
    ;**********************************************************************                                             
POS_EDGE_INT:                                                                                       
    clr     tmp                         ; Only clear the timer if this was a pos interrupt
    out     TCNT0, tmp                  ; Reset timer value if pos_edge_int               

    ldi     tmp, RX_TIMER_SYNC_MIN       
    cp      pulse_length, tmp    
    brlo    NOISE_DETECTED 
                                                              
    sbrc    rf_status, FRM_FLAG         ; Now SOF has been detected from previous neg. interrupt and not sampling data yet
    rjmp    RX_DATA_INIT                 
    
    sbrc    rf_status, RD_FLAG
    rjmp    RX_DATA
                  
RX_SYNC_SEARCH:                         ; Default
    ;**********************************************************************
    ; Check timing - Track and compare
    ;**********************************************************************     
    mov     pulse_tmp, pulse_length     ; Save time information  
 COMP_PULSE:
    rcall   CORR_PULSE          
    breq    SAME_SYNC_PULSE
 
    ;**********************************************************************
    ; 
    ;**********************************************************************     
SYNC_OUT_OF_RANGE:                      ; if "INSYNC" then RESTART SYNC_SEARCH    
SET_POS_EDGE:                           ; No matter what the next interrupt shall be on pos. edge    
    in      tmp, MCUCR                  ; Read MCU control register
    sbr     tmp, 1<<ISC01 | 1<<ISC00    ; This shall always be set (done here just in case becuse it is reset in power down)
    out     MCUCR, tmp                  ; Write back value (setting new trigger edge)             
    sbrc    rf_status, SOF_FLAG         
    rjmp    NEG_EDGE_NO_SOF             ; ?>> This was a negative interrupt that shall not be considered 
                                        ; (pos. period of SOF not found)

;OUT_OF_SYNC: 
    ldi     tmp, RX_SYNC_CNT            ; Reset the sync counter (look for sync from the beginning again)
    mov     rf_sync_cnt, tmp
    rjmp    STORE_CHIP_TIMING           ; >>>    
                                                                           
                                                                                                                                                                                                                                          
    ;********************************************************************** 
    ; 
    ;**********************************************************************
SYNC_SEARCH:    
    cbr     rf_status, (1<<SYNC_FLAG | 1<<SOF_FLAG)
    rjmp    STORE_CHIP_TIMING           ; >>>
    
        
    ;********************************************************************** 
    ; 
    ;**********************************************************************     
SAME_SYNC_PULSE:                        ; Check if in-sync                                         
    dec     rf_sync_cnt
    brne    SYNC_SEARCH    
IN_SYNC:    
    sbrc    rf_status, SOF_FLAG         ; If SOF_FLAG set then this was a neg. interrupt and first part of SOF found
    rjmp    POS_SOF_DETECTED    
    inc     rf_sync_cnt                 ; If "IN_SYNC" inc. to keep at zero           
    sbr     rf_status, (1<<SYNC_FLAG | 1<<SOF_FLAG) ; We are in sync so look for SOF in the next neg edge int
    
SET_NEG_EDGE:                           ; Now there was a pos. interrupt and IN_SYNC enable neg_edge_int for next time
    in      tmp, MCUCR                  ; Read MCU control register
    sbr     tmp, 1<<ISC01               ; This shall always be set (done here just in case becuse it is reset in power down)
    cbr     tmp, 1<<ISC00               ; If high the set low
    out     MCUCR, tmp                  ; Write back value (setting new trigger edge)
    rjmp    STORE_CHIP_TIMING           ; >>>
                                            
   
    ;**********************************************************************
    ; 
    ;**********************************************************************     
NOISE_DETECTED:           
    dec     idle_timer                            
    brne    STORE_CHIP_TIMING
    rjmp    RX_INIT
        
STORE_CHIP_TIMING:               
    mov     pulse_ref, pulse_length     ; Restore from stack and save time information of last pulse        
    rjmp    END_OF_EXT_INT0
                                                                           
                                                                            
    ;**********************************************************************
    ; 
    ;**********************************************************************
POS_SOF_DETECTED:  
    ; Set POS edge
    in      tmp, MCUCR                  ; Read MCU control register
    sbr     tmp, 1<<ISC01 | 1<<ISC00    ; This shall always be set (done here just in case because it is reset in power down)
    out     MCUCR, tmp                  ; Write back value (setting new trigger edge)            
    
    ; Calculate timing for neg. level of SOF
    ; it can be 4 or 5 chip periods depending on level of first data bit
    ; in addition there will be 4 chip periods caused by the high level SOF (since last pos int)

    cbr     rf_status, 1<<SOF_FLAG      
    sbr     rf_status, 1<<FRM_FLAG      ; Set Frame flag this is frame info          
    rjmp    END_OF_EXT_INT0


    ;**********************************************************************
    ; Clean up and return
    ;********************************************************************** 
NEG_EDGE_NO_SOF:                                                                       
    cbr     rf_status, 1<<SOF_FLAG      ; This was not the SOF so clear the flag
END_OF_EXT_INT0:            
    ldi     tmp, 1<<INT0                ; Clear any pending interrupts     
    out     GIFR, tmp                                      
    reti
 

    ;**********************************************************************
    ; 
    ;**********************************************************************     
RX_DATA_INIT:                   
    cbr     rf_status, (1<<SOF_FLAG | 1<<FRM_FLAG)  ; No longer getting SOF  
    sbr     rf_status, (1<<RD_FLAG | 1<<CHIP_FLAG)  ; Set from zero to "1" Now indicating that we are receiving data
       
    ldi     ZL, RF_DATA_BUFFER          ; Setup pointer to first byte in SRAM (data buffer)
    ldi     rf_bit_cnt, 0x80            ; pointing at MSB first since this is received first    
    clr     rf_buffer                   ; Clear the buffer                                                                                
    
    ; =====================================================================
    ;   Now the last neg. edge after of SOF has been found.
    ;   Determine polarity of first data bit depending on pulse_length. 
    ;   This will be 8 or 9 CHIP periods. 
    ; 
    ;           |  Get timing   |<-- 8 CHIPS -->|    
    ;           |  Get timing   |<-- 9 CHIPS ---->|  
    ;    _   _   _   _   _   _   _______                         
    ;   / \_/ \_/ \_/ \_/ \_/ \_/       \_______/./-X-x-X-x-X-x-X
    ;           |               |               |
    ;   |   |   |   |   |   |   |   |   |   |   |   |   |   |   |
    ;      .............. SYNC  |      SOF      |  POLARITY OF FIRST BIT    
    ;
    ; =====================================================================         
    mov     pulse_tmp, pulse_length
    lsr     pulse_tmp                   ; Now divide SOF by 4 to compare it     
    lsr     pulse_tmp                   ; with the standard 2 CHIP bit periode
    
    rcall   CORR_PULSE                  ; The 8 CHIPS will be recognized and 9 CHIPS will be out of range
    breq    FIRST_BIT_STORE_ONE
FIRST_BIT_STORE_ZERO:                        
    cbr     rf_status, 1<<CHIP_FLAG 
FIRST_BIT_STORE_ONE:    
    rcall   RX_STORE_BIT                ; Store first bit
    reti

    ;**********************************************************************
    ; 
    ;**********************************************************************
RX_DATA: 
    mov    pulse_tmp, pulse_length      ; Save time information                      
    rcall   CORR_PULSE    
    brne    BIT_OUT_OF_RANGE
                                                                                                                                                                                                                                 
    ;**********************************************************************
    ; 
    ;**********************************************************************
TWO_CHIP:                 
    ; 2 CHIP periodes is corresponding to one bit periode   
    ; Save a bit with the same polarity as last bit. 
    ; The initial value is '1' and no CHIP pending
    rcall   RX_STORE_BIT                ; Using same CHIP as last pass
    reti
                                                                                                                                                       
    ;**********************************************************************
    ; 
    ;**********************************************************************                                                                                  
BIT_OUT_OF_RANGE:                   
    ; Now we have either found 3 CHIP periodes or EOP.
    ; If CHIP_FLAG is pending then store two bit with the inverted value from last value of T and clear it.
    lsr     pulse_length                ; Divide by 2
    mov     pulse_tmp, pulse_length     
    rcall   CORR_PULSE                  ; Correlate against 4 CHIP then check for EOP   
    brne    THREE_CHIP_OR_EOP
                                                                                                                                     
FOUR_CHIP:   
    sbr     rf_status, 1<<CHIP_FLAG
    rcall   RX_STORE_BIT                ; Store '1'
    cbr     rf_status, 1<<CHIP_FLAG
    rcall   RX_STORE_BIT                ; Store '0'
    reti

THREE_CHIP_OR_EOP:                     
    ; NOW = X,  2 CHIP < X > 4 CHIP < X      
    ; =================================     
    cp      pulse_ref, pulse_length     ; Check if pulse_length >= 4 CHIP periodes
    brlo    EOP_DETECTED               

    ; NOW 3 CHIPS: Write '0' if CHIP = '1' else write "11"
    ; ====================================================
    sbrc    rf_status, CHIP_FLAG
    rjmp    PUT_SINGLE_ZERO             ; ...if CHIP_FLAG = '1'

    sbr     rf_status, 1<<CHIP_FLAG
    rcall   RX_STORE_BIT                ; Store '1'
    rcall   RX_STORE_BIT                ; Store '1'
    reti
        
PUT_SINGLE_ZERO:    
    cbr     rf_status, 1<<CHIP_FLAG
    rcall   RX_STORE_BIT                ; Store '0'
    reti                                                                           
                                                                            
    ;**********************************************************************
    ; 
    ;**********************************************************************    
EOP_DETECTED:                                      
    sbr     rf_status, 1<<RDR_FLAG
    rjmp    RX_INIT                     



;***************************************************************************
;
;    Procedure:         RX_STORE_BIT
;    
;    Description:       Write one bit (same as T in SREG) into rf_buffer[rf_bit_cnt]
;                       and store in RAM if rf_buffer full
;
;    Varibles affected: rf_buffer, rf_bit_cnt, RAM
;
;***************************************************************************              
RX_STORE_BIT:    
    sbrc    rf_status, CHIP_FLAG        ; If CHIP_flag was cleared then don't set corresponding bit in rf_buffer
    or      rf_buffer, rf_bit_cnt       ; Else set the corresponding bit                    
           
WRITE_TO_RAM_BUFFER:    
    lsr     rf_bit_cnt                  ; Shift bit pointer one position to the right    
    brne    EXIT_RX_DATA                ; Don't write to rx buffer
                         
    ; First check if Z pointer within range                         
    cpi      ZL, (RF_DATA_BUFFER + RF_DATA_BUFFER_SIZE)
    brlo    STORE_RX_BYTE
    ldi     ZL, RF_DATA_BUFFER          ; Reset pointer (wrap around in data buffer)
        
STORE_RX_BYTE:     
    inc     rf_data_cnt
                                  
    ; Load correct bytes into RAM
    st      Z+, rf_buffer               ; Store and post increment                            
    clr     rf_buffer                   ; clear buffer
    ldi     rf_bit_cnt, 0x80            ; Reset pointer    
     
EXIT_RX_DATA:
    ret    
 


;***************************************************************************
;
;    Procedure:         CORR_PULSE
;    
;    Description:       Correlate pulse_tmp against pulse_ref
;
;    Parameters:        tmp -> (maximum number)-1 of bits offset
;                       pulse_tmp
;
;    Varibles affected: pulse_tmp, tmp, pulse_ref (not changed)
;
;    Return:            Z flag = 1: SAME_LENGTH, Z flag = 0: OUT_OF_RANGE
;                       (pulse_tmp -> adjusted upto tmp-1 up or down against pulse_ref)
;                       tmp -> indicate offset, if reaching same pulse before reaching zero
;
;*************************************************************************** 
CORR_PULSE:                      
    ldi     tmp, PULSE_OFFSET           ; Offset = tmp-1 Minimum number is 2 = +/-1, 3 = +/-2.... 
                                        ; Now looping (tmp-1) number of times before giving up
COMPARE_PULSE:    
    dec     tmp
    breq    PULSE_OUT_OF_RANGE          ; >>> If function not resolved yet then this CHIP must be out of range    
    cp      pulse_ref, pulse_tmp        ; Compare low byte to se if higher or lower than last interrupt
    breq    SAME_PULSE                  ; >>> OK - CHIP within range
    brlo    SHORTER_PULSE               ; Now the values are higer or lower; jump if lower (shorter pulse)
LONGER_PULSE:                           ; Dont jump if higher (longer pulse)
    inc     pulse_tmp                                                                   
    rjmp    COMPARE_PULSE
SHORTER_PULSE:                
    dec     pulse_tmp
    rjmp    COMPARE_PULSE                        
    
SAME_PULSE:  
    sez                                 ; Set zero
    ret 

PULSE_OUT_OF_RANGE:    
    clz                                 ; Clear zero flag
    ret
    
 
                                                                                                              
;***************************************************************************
;
;    Procedure:         SETUP_CC400_TX
;    
;    Description:       Write 8 commands from EEPROM to CC400
;
;    Varibles affected: eep_buffer, tmp, eep_pt, rf_bit_cnt
;
;***************************************************************************
SETUP_CC400_TX:                         
    cli   
    ldi     rf_bit_cnt, 8               ; Counter of 8 bytes    
    ldi     tmp, TX                     ; Set pointer to "TX" string in EEPROM
    mov     eep_pt, tmp
    rcall   CC400_CMD                   ; This routine will use the two parameters above
    sei
    ret

           
            
;***************************************************************************
;
;    Procedure:         SETUP_CC400_RX_RECHARGE
;    
;    Description:       Write 8 commands from EEPROM to CC400
;
;    Varibles affected: eep_buffer, tmp, eep_pt, rf_bit_cnt
;
;***************************************************************************
SETUP_CC400_RX_PC:      
    cli    
    ldi     rf_bit_cnt, 8
    ldi     tmp, RX_PC
    mov     eep_pt, tmp
    rcall   CC400_CMD
    sei
    ret
 


;***************************************************************************
;
;    Procedure:         SETUP_CC400_RX
;    
;    Description:       Setup CC400 to RX mode after RX_PRECHARGE by writing on command (16 bits) from EEPROM to CC400
;
;    Varibles affected: eep_buffer, tmp, eep_pt, rf_bit_cnt
;
;***************************************************************************
SETUP_CC400_RX:      
    cli
    ldi     rf_bit_cnt, 1
    ldi     tmp, RX     
    mov     eep_pt, tmp    
    rcall   CC400_CMD
    sei
    ret



;***************************************************************************
;
;    Procedure:         SETUP_CC400_IDLE
;    
;    Description:       Setup CC400 to Power down mode 2 words, with oscillator active
;
;    Varibles affected: eep_buffer, tmp, eep_pt, rf_bit_cnt
;
;***************************************************************************
SETUP_CC400_IDLE:      
    cli    
    ldi     rf_bit_cnt, 2
    ldi     tmp, PD                     
    mov     eep_pt, tmp 
    rcall   CC400_CMD
    sei
    ret



;***************************************************************************
;
;    Procedure:         SETUP_CC400_PD
;    
;    Description:       Setup CC400 to Power down mode 2 words
;
;    Varibles affected: eep_buffer, tmp, eep_pt, rf_bit_cnt
;
;***************************************************************************
SETUP_CC400_PD:      
    cli
    ldi     rf_bit_cnt, 2
    ldi     tmp, PD 
    mov     eep_pt, tmp        
    rcall   CC400_CMD
    sei
    ret
 
 

;***************************************************************************
;
;    Procedure:         Write word including strobe to CC400
;    
;    Description:       Write one 16 bit word from EEPROM to CC400
;
;    Varibles affected: eep_buffer, tmp, eep_pt, rf_bit_cnt
;
;   eep_pt shall be set to correct initialize value before entering this routine
;
;***************************************************************************
CC400_CMD:
    cbi     PORTB, STROBE               ; Make strobe floating or low level (just in case)
    sbi     DDRB, STROBE                ; Strobe is now active low output (just in case)

    sbi     DDRB, PDATA                 ; Set data line to output
                                        ; Data value will be updated in the following subroutines
    
    cbi     PORTB, CLOCK                ; Make clock floating or low level (just in case)
    sbi     DDRB, CLOCK                 ; Clock is now active low output (just in case)
            
    rcall   CC400_WORD                
    inc     eep_pt                      
    dec     rf_bit_cnt
    brne    CC400_CMD
                                                       

    ; Set both data and clock as inputs in case these shall be used for other purposes as well
    ;sbi     PORTB, PDATA      ; Set high (high fanout)                 <-- Disable this command. Required by the COM interface
    ;cbi     DDRB, PDATA       ; Input with pull-up (low fanout)         <-- Disable this command. Required by the COM interface
                                           
    sbi     PORTB, CLOCK                ; Set high (high fanout)
    cbi     DDRB, CLOCK                 ; Input with pull-up (low fanout)
    ret
           
            

;***************************************************************************
;
;    Procedure:         Write word including strobe to CC400
;    
;    Description:       Write one 16 bit word from EEPROM to CC400
;
;    Varibles affected: eep_buffer, tmp
;
;***************************************************************************
CC400_WORD:    
    cbi     PORTB, STROBE               ; Set strobe and clock signals low
    cbi     PORTB, CLOCK
        
    rcall   CC400_BYTE                  ; If defining word in eseg then => inc. eep_pt
        
    ;if defining word in eseg then => dec. eep_pt and remove next line
    inc     eep_pt 
    rcall   CC400_BYTE

    sbi     PORTB, STROBE               ; Make a strobe pulse
    nop    
    cbi     PORTB, STROBE                                                           
    ret



;***************************************************************************
;
;    Procedure:         Write byte to CC400
;    
;    Description:       Write one 16 bit word from EEPROM to CC400
;
;    Varibles affected: eep_buffer, tmp
;
;*************************************************************************** 
CC400_BYTE:
    rcall   EEGET                       ; Read EEPROM value into eep_buffer    
    ldi     tmp, 8                      ; Preset bit counter variable

CC400_BYTE_LOOP:
                                
    ; Set PDATA pin to correct value                                 
    sbrc    eep_buffer, 7
    sbi     PORTB, PDATA  
    sbrs    eep_buffer, 7
    cbi     PORTB, PDATA
    
    lsl     eep_buffer                  ; Shift left

    ; Make a clock pulse
    sbi     PORTB, CLOCK
    nop
    cbi     PORTB, CLOCK

    dec     tmp                         ; Dec bit counter
    brne    CC400_BYTE_LOOP    
    ret



;***************************************************************************
;
;    Procedure:         Get EEPROM value
;    
;    Description:       Read byte from location pointed to by eep_pt
;                       and return value in eep_buffer.
;
;    Varibles affected: eep_buffer
;
;***************************************************************************
EEGET:
   sbic     EECR, EEWE
   rjmp     eeget
   
   out      EEAR, eep_pt
   sbi      EECR, EERE

poll_data_ready:   
   sbic     EECR, EERE
   rjmp     poll_data_ready
   
   in       eep_buffer, EEDR 
   ret



;***************************************************************************
;
;    Procedure:         Set EEPROM value
;    
;    Description:       Write byte from eep_buffer to location pointed to by eep_pt.
;                       
;    Varibles affected: none
;
;***************************************************************************
EEPUT:                         
   sbic     EECR, EEWE
   rjmp     eeput
   
   out      EEAR, eep_pt                ; Write address
   out      EEDR, eep_buffer            ; Write data
   sbi      EECR, EEWE 
   ret



;***************************************************************************
;
;    Enter POWER_DOWN mode
;
;***************************************************************************    
POWER_DOWN:                         
    cli                                 ; Disable all interrupts                              
     
    rcall   SETUP_CC400_PD

    ; Make I/O-pin for ON_BUTTON  input with pull-up, first active high
    sbi     DDRB, ON_BUTTON             ; Output
    sbi     PORTB, ON_BUTTON            ; High
    cbi     DDRB, ON_BUTTON             ; Input with pull-up
                                                                               

CHECK_INT0_LINE:                        ; Wait until DIO ON_BUTTON line has gone high
    sbis    PINB, ON_BUTTON             ; LED_ON 
    rjmp    CHECK_INT0_LINE
    LED_OFF                             ; Now the ON button is released then turn off the LED
                                                                                                                                                                           
                                        ; A low level external interrupt or a watchdog timeout (if watchdog is enabled)
                                        ; will reset the controller. This is the only way to continue execution from here.                               
    wdr                                 ; watchdog reset just in case    
    ldi     tmp, (1<<WDTOE | 1<<WDE | 1<<WDP2 | 1<<WDP1 | 0<<WDP0)    
    out     WDTCR, tmp                  ; Turn off watchdog part #1       
    ldi     tmp, (1<<WDTOE | 0<<WDE | 1<<WDP2 | 1<<WDP1 | 0<<WDP0)
    out     WDTCR, tmp                  ; Turn off watchdog part #2
                                                     
    clr     tmp
    out     GIMSK, tmp                  ; Disable EXTERNAL interrupt before changing to low level trigger                

    ldi     tmp, 1<<SE | 1<<SM | 0<<ISC01 | 0<<ISC00
    out     MCUCR, tmp                  ; Set external interrupt to low level trigger: D0 = '0' D1 = '0'                
                                        ; Set SE bit to enable sleep mode and SM bit to select power down
     
    ser     tmp                         ; same as: ldi     tmp, 1<<INT0
    out     GIMSK, tmp                  ; enable external interrupt
    sei                                 ; Enable interrupt
    sleep                               ; Put MCU in IDLE mode                         
                                         
    rjmp reset                          ; This will simply re-init MCU (reset stack pointer etc.)



;***************************************************************************
;
;    Clear all registers
;
;***************************************************************************    
CLEAR_ALL_REGISTERS:
    clr    r0
    clr    r1
    clr    r2
    clr    r3
    clr    r4
    clr    r5
    clr    r6
    clr    r7
    clr    r8
    clr    r9
    clr    r10
    clr    r11
    clr    r12
    clr    r13
    clr    r14
    clr    r15    
    clr    r16
    clr    r17
    clr    r18    
    clr    r19
    clr    r20
    clr    r21
    clr    r22
    clr    r23
    clr    r24
    clr    r25
    clr    r26
    clr    r27
    clr    r28
    clr    r29
    clr    r30
    clr    r31    
    ret
                               
.exit

